@@ -2,15 +2,10 @@
 # For more info, including license, please see doc/index.html
 
 package Edge;
-# Edge of a Graph
 
 use strict;
-use Carp;
-use vars qw(@ISA);
-@ISA = qw(Configurable);
-
-use Configurable;
-use Vector2;
+use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
+use Vector;
 use overload
     '""' => 'stringify',
     'fallback' => 1
@@ -18,20 +13,37 @@ use overload
 #    'fallback' => undef
 ;
 
+require Exporter;
+use Carp;
+
+@ISA = qw(Exporter);
+@EXPORT = qw();
+@EXPORT_OK = qw();
+%EXPORT_TAGS = (all => [@EXPORT_OK]);
+
 sub new {
     my ($class, $src, $tgt, %opts) = @_;
     $class = ref($class) if ref($class);
+    %opts = %{ ::merge_config_opts($class, \%opts) };
+#print join(",",%$opts), "\n";
     croak "$src is not a Vertex" unless $src->isa("Vertex");
     croak "$tgt is not a Vertex" unless $tgt->isa("Vertex");
-    my ($self) = bless ::deep_copy(\%opts), $class;
-    $self->{"#host"} = $src->host();
-    my ($cv) = $self->host()->cget(-canvas);
-    $self->{shape_id} = $cv->createLine(0, 0, 0, 0);
-    $self->{text_id} = $cv->createText(0, 0);
+    my ($self) = bless { # default values
+	adj => {
+	    src => $src,
+	    tgt => $tgt,
+	},
+    }, $class;
+    my ($k, $v);
+    # the following options need only be stored and need no further processing
+    foreach $k (qw(-host -weight -directed)) {
+	$self->{$k} = exists $opts{$k} ? delete $opts{$k} : undef;
+    }
+    $self->{shape_id} = $self->{-host}{-canvas}->createLine(0, 0, 0, 0);
+    $self->{text_id} = $self->{-host}{-canvas}->createText(0, 0);
+    $self->set_ends($src, $tgt, -directed=>$self->{-directed});
+    $self->configure(-arrow=>$self->{-directed} ? "last" : "none", %opts);
 #    $self->{canvas}->addtag($name, "withtag", $self->{obj}{$name}{shape_id});
-    $self->set_ends($src, $tgt);
-    $self->{-arrow} = $self->cget(-directed) ? "last" : "none";
-    $self->configure($self->get_all_opts());
     return $self;
 }
 
@@ -43,7 +55,7 @@ sub stringify {
 
 sub destroy {
     my ($self) = @_;
-    $self->host()->cget(-canvas)->delete(@{$self}{"shape_id","text_id"});
+    $self->{-host}{-canvas}->delete(@{$self}{"shape_id","text_id"});
 }
 
 sub source {
@@ -58,23 +70,20 @@ sub target {
     return $self->{adj}{tgt};
 }
 
-sub host {
-    return $_[0]->{"#host"};
-}
-
 # intersection of ellipse (x/a)^2 + (y/b)^2 = 1 with line x/dx = y/dy
 sub _x_oval_ {
     my ($dx, $dy, $a, $b) = @_;
     $a = $dx / $a;
     $b = $dy / $b;
     my ($t) = sqrt($a*$a + $b*$b);
-    return Vector2->new($dx/$t, $dy/$t);
+    return Vector->new($dx/$t, $dy/$t);
 }
 
 sub set_ends {
     my ($self, $src, $tgt, %opts) = @_;
-    my ($cv) = $self->host()->cget(-canvas);
-    @{ $self->{adj} }{qw(src tgt)} = ($src, $tgt);
+    return unless defined $self->{-host}{-canvas};
+    $src = $self->{adj}{src} unless defined $src;
+    $tgt = $self->{adj}{tgt} unless defined $tgt;
     my ($pos_s, $size_s) = $src->_get_cv_geom_();
     my ($pos_t, $size_t) = $tgt->_get_cv_geom_();
     $size_s = $size_s->pw_div(2);
@@ -85,29 +94,30 @@ sub set_ends {
 	$pos_t -= _x_oval_( @$d, @$size_t);
     } else {
 	carp "Both ends of edge $self coincide (at $pos_s)\n"
-	    unless $self->cget(-quiet);
+	    unless $opts{-quiet};
     }
-    # now turn $d 90 degrees counter-clockwise and make it a unit vector
-    @$d = (-$d->[1], $d->[0]);
-    $d = $d->pw_div($d->norm());
-    my ($s, $t);
-    $s = $d->pw_mul($self->cget(-directed) ? 4 : 0);
-    $pos_s += $s; $pos_t += $s;
-    $cv->coords($self->{shape_id}, @$pos_s, @$pos_t);
-    $s += $d->pw_mul(8);
-    $s = $s->pw_mul(-1) if (not $self->cget(-directed) and
-	$s->[0] < 0 or $s->[0] == 0 and $s->[1] < 0);
-    $t = $self->cget(-directed) ? 2/5 : 1/2;
-    $cv->coords($self->{text_id},
-	@{ $pos_s->pw_mul($t) + $pos_t->pw_mul(1-$t) + $s }
+    my ($t, $s);
+    if ($opts{-directed}) {
+	$t = 1/3;
+	$s = 4/$d->norm();
+    } else {
+	$t = 1/2;
+	$s = 0;
+    }
+    @{$d}[0,1] = (-$d->[1]*$s, $d->[0]*$s);
+    $pos_s += $d; $pos_t += $d;
+    $self->{-host}{-canvas}->coords($self->{shape_id}, @$pos_s, @$pos_t);
+    $self->{-host}{-canvas}->coords($self->{text_id},
+	@{ $pos_s->pw_mul($t) + $pos_t->pw_mul(1-$t) }
     );
 }
 
 sub configure {
     my ($self, %opts) = @_;
+    return unless defined $self->{-host}{-canvas};
     my ($k, %shape_opts, %text_opts);
     my ($opt_map) = {
-	-weight   => [undef, undef],
+#	-state   => ["-state", "-state"],
 	-text    => [undef, "-text"],
 	-width	 => ["-width", undef],
 	-fill    => [undef, undef],
@@ -120,14 +130,13 @@ sub configure {
     };
     if (exists $opts{-directed}) {
 	$self->{-directed} = delete $opts{-directed};
-	$self->set_ends($self->source(), $self->target());
+	$self->set_ends($self->source(), $self->target(), -directed=>$self->{-directed});
     }
     if (exists $opts{-status}) {
 	$self->{-status} = delete $opts{-status};
-	my ($ha) = $self->host()->cget(-appearance);
 	carp "unknown status $self->{-status} ignored"
-	    unless exists $ha->{$self->{-status}};
-	%opts = (%{ $ha->{$self->{-status}} }, %opts);
+	    unless exists $self->{-host}{-appearance}{$self->{-status}};
+	%opts = (%{ $self->{-host}{-appearance}{$self->{-status}} }, %opts);
     }
     foreach $k (keys %opts) {
 	carp "unknown option $k ignored" unless exists($opt_map->{$k});
@@ -136,21 +145,15 @@ sub configure {
 	$text_opts{ $opt_map->{$k}[1] } = $opts{$k}
 	    if defined $opt_map->{$k}[1];
     }
-    my ($cv) = $self->host()->cget(-canvas);
-    $cv->itemconfigure($self->{shape_id}, %shape_opts);
-    $cv->itemconfigure($self->{text_id}, %text_opts);
+    $self->{-host}{-canvas}->itemconfigure($self->{shape_id}, %shape_opts);
+    $self->{-host}{-canvas}->itemconfigure($self->{text_id}, %text_opts);
 }
 
-sub get_all_opts {
-    my ($self) = @_;
-    my (%opts) = $self->SUPER::get_all_opts();
-    delete @opts{qw(-display -shape -size -name)};
-    return %opts;
+sub cget {
+    my ($self, $opt) = @_;
+    return exists $self->{$opt} ? $self->{$opt} :
+	$self->{-host}{-canvas}->itemcget($opt);
 }
 
-$::Config->{Edge} = {
-    -status => "init",
-};
-
 1;
 
