@@ -2,19 +2,6 @@
 # For more info, including license, please see doc/index.html
 
 package Graph;
-# Graph
-
-use strict;
-use Carp;
-use vars qw(@ISA);
-@ISA = qw(Collection);
-
-use Collection;
-use Vertex;
-use DCEdge;
-# { package main; require "graph/dfs"; }
-
-# use Data::Dumper;
 
 # We don't distinguish abstract graphs from geometric graphs.
 # Every graph is given the DCEL structure. This makes the logic
@@ -22,17 +9,42 @@ use DCEdge;
 # Every graph is directed, and every edge is paired. Artificial
 # edges are hidden.
 
+use strict;
+use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
+use Vector;
+use Vertex;
+use DCEdge;
+{ package main; require "graph/dfs"; }
+
+require Exporter;
+use Carp;
+
+@ISA = qw(Exporter);
+@EXPORT = qw();
+@EXPORT_OK = qw();
+%EXPORT_TAGS = (all => [@EXPORT_OK]);
+
+$VERSION = '0.1';
+
+# use Data::Dumper;
+
 sub new {
     my ($class, %opts) = @_;
     $class = ref($class) if ref($class);
     my ($init_data) = delete $opts{-init_data};
     my ($operation) = delete $opts{-operation};
+    %opts = %{ ::merge_config_opts($class, \%opts) };
     my ($t) = delete $opts{-type};
     croak "'Graph' code does not know how to process '$t' data\n"
 	unless ($t eq 'graph' or $t eq 'points');
-    my ($self) = $class->SUPER::new(%opts);
+    my ($self) = bless { }, $class;
 
     my (@v_names, $k, $s);
+#print STDERR Dumper($data), "\n";
+    # the following options need only be stored and need no further processing
+    foreach $k (qw(-name -canvas -appearance -linear_transform -directed -node_opts)) {
+	$self->{$k} = exists $opts{$k} ? delete $opts{$k} : undef;
+    }
 
     if ($t eq 'points') {
 	my ($n, $i, $name) = $#$init_data;
@@ -74,8 +86,8 @@ sub new {
 	@nbr = sort grep { not /\W/ } keys %{$init_data->{$s}};
 	foreach $t (@nbr) {
 	    $n2E->{$s}{$t} = DCEdge->new($n2V->{$s}, $n2V->{$t},
-		-weight=>$init_data->{$s}{$t}, -text=>$init_data->{$s}{$t},
-		-directed=>$self->cget(-directed)
+		-host=>$self, -weight=>$init_data->{$s}{$t},
+		-text=>$init_data->{$s}{$t}, -directed=>$self->{-directed}
 	    );
 	}
 	$self->{"#OneEdge"} = $n2V->{$s}->{"#OneEdge"} =
@@ -88,9 +100,17 @@ sub new {
     # edges are refered to as objects (blessed references). Code
     # readers are reminded to think in object terms.
 
+    my (@unknown) = keys %opts;
+    carp "unknown option(s) '@unknown' ignored" if (@unknown);
     return $self;
 }
 
+sub cget {
+    my ($self, $opt) = @_;
+    carp "unknown option '$opt' ignored" unless exists $self->{$opt};
+    return $self->{$opt};
+}
+
 sub dcelify {
 # build Doubly Connected Edge List pointers
     my ($self, $n2V, $n2E) = @_;
@@ -100,9 +120,10 @@ sub dcelify {
     foreach $s (keys %$n2E) {
 	foreach $t (keys %{ $n2E->{$s} }) {
 	    if (not exists $n2E->{$t}{$s}) {
-		$n2E->{$t}{$s} = DCEdge->new($n2V->{$t}, $n2V->{$s}, -arrow=>"last");
-		if ($self->cget(-directed)) {
-		    $n2E->{$t}{$s}->phantomize();
+		$n2E->{$t}{$s} = DCEdge->new($n2V->{$t}, $n2V->{$s},
+		    -host=>$self, -arrow=>"last");
+		if ($self->{-directed}) {
+		    $n2E->{$t}{$s}->configure(-status=>"phantom");
 		} else {
 		    warn "one way edge detected";
 		    $n2E->{$t}{$s}->configure(-status=>"alert", -directed=>1);
@@ -111,7 +132,7 @@ sub dcelify {
 	    $n2E->{$s}{$t}->twin($n2E->{$t}{$s});
 	    $n2E->{$t}{$s}->twin($n2E->{$s}{$t});
 	    my ($w) = $n2E->{$t}{$s}->cget(-weight);
-	    if (not $self->cget(-directed)) {
+	    if (not $self->{-directed}) {
 		if (not defined $w or $w != $n2E->{$s}{$t}->cget(-weight)) {
 		    warn "$s-$t is different from $t-$s in an undirected graph";
 		    $n2E->{$t}{$s}->configure(-status=>"alert",
@@ -127,8 +148,7 @@ sub dcelify {
 	my ($src_pos) = $n2V->{$s}->pos();
 	foreach $t (keys %{ $n2E->{$s} }) {
 	    my ($tgt_pos) = $n2V->{$t}->pos();
-	    my ($a) = $tgt_pos - $src_pos;
-	    push @$neighbor, {name=>$t, angle=>atan2($a->[1], $a->[0])};
+	    push @$neighbor, {name=>$t, angle=>($tgt_pos-$src_pos)->angle()};
 	}
 	next unless $#$neighbor >= 0;
 	$neighbor = [ map { $_->{name} }
@@ -155,11 +175,10 @@ sub dcelify {
 
 sub v_new {
     my ($self, $name, %opts) = @_;
-    %opts = ( %{ $self->cget(-node_opts) }, %opts );
-    # as always, the host should take care of prepending %opts with -node_opts
-    my ($pos) = Vector2->new(@{ delete $opts{-pos} });
-    my ($v) = Vertex->new($self, $pos, -name=>$name, %opts);
-#    $v->configure(-text=>$v->cget(-display)->($v)) unless defined $opts{-text};
+    %opts = %{ ::merge_config_opts($self->{-node_opts}, \%opts) }
+	if defined $self->{-node_opts};
+    my ($v) = Vertex->new(-host=>$self, -name=>$name, %opts);
+    $v->configure(-text=>$v->{-display}->($v)) unless defined $opts{-text};
     return $v;
 }
 
@@ -185,7 +204,7 @@ sub edges_around {
     croak "broken 'OneEdge' on vertex $v" unless $e->source() eq $v;
     $start = $e;
     do {
-	push @s, $e unless $e->is_phantom();
+	push @s, $e unless $e->cget(-status) eq "phantom";
 	$e = $e->prev()->twin();
     } while ($e ne $start);
     return @s;
@@ -193,14 +212,39 @@ sub edges_around {
 
 sub destroy {
     my ($self) = @_;
-    ::pfs($self, $self->cget(-canvas),
-	-priority=>"sbs",
+    ::dfs($self,
 	-on_vertex=>sub { $_[0]->destroy(); },
 	-on_edge=>sub { $_[0]->destroy(); },
     );
     undef $self;
 }
 
+#sub vertices {
+#    return @{ $_[0]->{"#Vertices"} };
+#}
+
+# sub get_all_vertices {
+# # This operation incurs a heavy cost and causes the status of
+# # vertices and edges to reset to "init", so give it a long name.
+# # Avoid calling it unless really necessary.
+#     my ($self) = @_;
+#     my (@V, $v, $e);
+#     ::dfs($self, -on_vertex=>sub { push @V, $_[0]; });
+#     foreach $v (@V) {
+# 	$v->configure(-status=>"init");
+# 	foreach $e ($self->edges_around($v)) {
+# 	    $e->configure(-status=>"init");
+# 	}
+#     }
+#     $self->{"#Vertices"} = [@V];
+#     return @V;
+# }
+
+#sub vertices {
+#    my ($self) = @_;
+#    my ($v) = $self->{"#OneEdge"}->source();
+#}
+
 if ($0 =~ /Graph.pm$/) {
 # being tested as a stand-alone program, so run test code.
 
@@ -218,9 +262,6 @@ sub disp_vert_val {
     $v->configure(-text=>"$v\n$val");
 }
 
-$::Config->{Graph} = {
-};
-
 #::dfs($g);
 
 # Theoretically, Dijkstra's single-source-shortest path algorithm should
@@ -251,13 +292,6 @@ print "$v: ", join(",", map {$_->target()} $g->edges_around($v)), "\n";
 #    print "\n";
 #}
 
-# $can->{main}->set_mark(1);
-$ctrl->configure(-recorder=>0);
-# If the canvas refuses to show any change, remember to verify that:
-# - set_mark() was called at least once
-# - -recorder is set to zero before entering MainLoop
-# Failing to do either of the above will result in a mysterious bug
-# that takes days to figure out !@#$%
 Tk::MainLoop();
 
 }
