@@ -2,24 +2,35 @@
 # For more info, including license, please see doc/index.html
 
 package PQueue;
-# Abstract Priority Queue
 
 use strict;
-use Carp;
-use vars qw(@ISA);
-@ISA = qw(Collection);
+use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
-use Collection;
+require Exporter;
+use Carp;
+use Data::Dumper;
 use overload
     '""' => 'stringify',
     'fallback' => undef
 ;
 
+@ISA = qw(Exporter);
+@EXPORT = qw();
+@EXPORT_OK = qw();
+%EXPORT_TAGS = (all => [@EXPORT_OK]);
+
+$VERSION = '0.1';
+
 sub new {
     my ($class, %opts) = @_;
     $class = ref($class) if ref($class);
-    my ($self) = $class->SUPER::new(%opts);
-    $self->{"#data"} = [0];
+    my ($updated_opt) = ::merge_config_opts($class, \%opts);
+    my ($self) = bless {"#data"=>[0]}, $class;
+    # the following options need only be stored and need no further processing
+    my ($k);
+    foreach $k (qw(-compare)) {
+	$self->{$k} = exists $opts{$k} ? delete $opts{$k} : undef;
+    }
 #    if (ref($class)) {
 #	$Data::Dumper::Terse = 1;
 #	return eval(Dumper($class));	# cloning
@@ -52,12 +63,10 @@ sub find {
     return $i;
 }
 
-$::Config->{PQueue} = {
-};
-
 if ($0 =~ /PQueue.pm$/) {
 # being tested as a stand-alone program, so run test code.
 
+require ".algotutor";
 my ($q) = PQueue->new();
 print "<$q>\n";
 
