@@ -2,12 +2,19 @@
 # For more info, including license, please see doc/index.html
 
 package Vector;
-# Mathematical Vector
 
 use strict;
+use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
+
+require Exporter;
 use Carp;
-use vars qw(@ISA);
-@ISA = qw();
+
+@ISA = qw(Exporter);
+@EXPORT = qw();
+@EXPORT_OK = qw();
+%EXPORT_TAGS = (all => [@EXPORT_OK]);
+
+$VERSION = '0.1';
 
 my (%generated);
 
@@ -50,29 +57,25 @@ BEGIN {
 
 # see perldoc overload, especially the "MAGIC AUTOGENERATION" section
 use overload
-    '='  => '_clone',
+    '='  => 'clone',
     '""' => 'stringify',
     '+'  => $generated{add},
     '-'  => $generated{sbt},
-    'neg'=> 'negate',
-    '*'  => $generated{mul},
-    '/'  => $generated{div},
+#    '*'  => $generated{mul},
+#    '/'  => $generated{div},
     'fallback' => undef
 ;
 
 sub pw_mul { return $generated{mul}->(@_); }
 sub pw_div { return $generated{div}->(@_); }
 
-# Different from "Perl Cookbook", chap 13.6, p.461 "cloning objects"
-# See Randal Schwartz's "Constructing Objects" at
-# http://www.stonehenge.com/merlyn/UnixReview/col52.html
-# (search for "three camps")
+
 sub new {
-    my ($proto, @data) = @_;
-    my ($class) = ref $proto || $proto;
+    my ($class, @data) = @_;
+    $class = ref($class) if ref($class);
 #    if (ref $data[0] eq "Vector") {
-    if (ref $proto) {
-	return bless [ @$proto ], $class;
+    if (ref $data[0]) {
+	return clone(undef, $data[0], "");
     } else {
 	return bless [@data], $class;
     }
@@ -82,9 +85,9 @@ sub new {
 # just before a mutator is applied to one of the reference
 # variables sharing the same copy. See perldoc overload,
 # especially the "Copy Constructor" section.
-sub _clone {
+sub clone {
     my ($a, $b, $switch) = @_;
-    print STDERR "Vector::_clone : switch is undef!\n"
+    print STDERR "Vector::clone : switch is undef!\n"
 	unless defined $switch;
     # print STDERR $switch ? "+" : "-"; # always prints "-"
     return $switch ? bless([@$a],"Vector") : bless([@$b],"Vector");
@@ -99,15 +102,6 @@ sub stringify {
     return $r . " ]";
 }
 
-sub negate {
-    my ($self) = @_;
-    return bless [map { -$_ } @$self], ref $self;
-}
-
-sub x { return $_[0]->[0]; }
-sub y { return $_[0]->[1]; }
-sub z { return $_[0]->[2]; }
-
 sub dot {
 # dot product
     my ($t) = $_[0]->pw_mul($_[1]);
@@ -132,7 +126,7 @@ sub cob {
 # change of basis
     my ($self, $b) = @_;
     die unless ($#$b == $#$self and $#$b == $#{$b->[0]});
-    my ($r) = $self->new();
+    my ($r) = Vector->new($self);
     map { $_ = 0; } @$r;
     my ($i);
     for ($i=0; $i<=$#$self; ++$i) {
@@ -141,18 +135,75 @@ sub cob {
     return $r;
 }
 
+### 2-d functions ###
+
+#sub slope {
+#    my ($self) = @_;
+#    return $self->[1]/$self->[0];
+#}
+
+sub angle {
+    my ($self) = @_;
+    return atan2($self->[1], $self->[0]);
+}
+
+sub orient2d {
+    my ($a, $b, $c) = @_;
+    $b -= $a; $c -= $a;
+    return $b->[0]*$c->[1] - $b->[1]*$c->[0];
+}
+
+sub makebasis {
+    my ($self) = @_;
+    my ($l) = $self->norm();
+    my ($c, $s) = ($self->[0]/$l, $self->[1]/$l);
+    return [Vector->new($c,-$s),Vector->new($s,$c)];
+}
+
+### 3-d functions ###
+
+sub cross {
+# cross product
+    my ($self, $other) = @_;
+    return Vector->new(
+	$self->[1]*$other->[2]-$self->[2]*$other->[1],
+	$self->[2]*$other->[0]-$self->[0]*$other->[2],
+	$self->[0]*$other->[1]-$self->[1]*$other->[0],
+    );
+}
+
+sub rotate_around {
+# rotate $self around $axis by $th
+    my ($self, $axis, $th) = @_;
+
+    my ($x, $r, $p);
+    $r = $axis->norm();
+    return $self if $r < 1e-5;
+    $x = $axis->pw_div($r);
+    $r = $x->pw_mul($self->dot($x));
+    $p = $self - $r;
+    return $r + $p->pw_mul(cos($th)) + $x->cross($p)->pw_mul(sin($th));
+}
+
 if ($0 =~ /Vector.pm$/) {
 # being tested as a stand-alone program, so run test code.
-    my ($p, $q, $r);
-    $p = Vector->new(4,-3);
-    $q = Vector->new(5,12);
-    print $p+$q, ",", $p-$q, "\n";
+    my ($x) = Vector->new(4,-3);
+    my ($y) = Vector->new(5,12);
+    my ($z);
 
-    $r = $p;
-    $r += $q;
-    $q = $q->pw_div(2);
-    print $p, ",", $q, ",", $r, ",", $p->pw_mul(3), ",", -$p, "\n";
+    print $x+$y, ",", $x-$y, ",",
+	$x->orient2d(Vector->new(0,0), $y), "\n";
+    $z = $x;
+    $z += $y;
+    $y = $y->pw_div(2);
+    print $x, ",", $y, ",", $z, ",", $x->pw_mul(3), "\n";
 
+    # 3d tests
+    $x = Vector->new(1,1,0);
+    $y = Vector->new(1,1,1);
+    $z = $x->rotate_around($y, atan2(1,1)*8/3);
+    print $z, "\n";
+    # should print [0, 1, 1]
 }
 
 1;
